/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2008 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.ui.channelclient;

import java.io.IOException;
import java.io.PrintWriter;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.ow2.petals.jmx.api.IPetalsMonitoringService.ChannelClientReportKeyNames;
import org.ow2.petals.tools.channelclient.jmxclient.ChannelClientJMXClient;
import org.ow2.petals.tools.channelclient.jmxclient.exception.ChannelClientDoesNotExistException;
import org.ow2.petals.tools.channelclient.jmxclient.exception.ChannelClientServiceErrorException;
import org.ow2.petals.tools.channelclient.jmxclient.exception.ConnectionErrorException;
import org.ow2.petals.tools.channelclient.jmxclient.exception.PerformActionErrorException;
import org.ow2.petals.tools.webadmin.cron.CollectorManager;
import org.ow2.petals.tools.webadmin.datacollector.client.CollectorClient;
import org.ow2.petals.tools.webadmin.datacollector.client.exception.DataCollectorClientException;
import org.ow2.petals.tools.webadmin.datacollector.server.CollectorServer;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorClientDoesNotExistException;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerException;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerNotInitializedException;
import org.ow2.petals.tools.webadmin.ui.infra.action.ChannelClientAction;
import org.ow2.petals.tools.webadmin.ui.infra.bean.InfoBean;
import org.ow2.petals.tools.webadmin.ui.infra.session.AdminSession;
import org.ow2.petals.tools.webadmin.ui.logon.LogonUtil;

/**
 * @author mcarpentier - EBM Websourcing
 * 
 * This class is an alternative to AJAX send message actions
 * 
 */
public class SendMessage extends ChannelClientAction {

    private static final String actionTitle = "send Message to PEtALS";

    final Logger log = Logger.getLogger(SendMessage.class);

     @Override
    public ActionForward executeChannelClientActionAuthentified(ActionMapping iActionMapping,
            ActionForm iActionForm, HttpServletRequest iServletRequest,
            HttpServletResponse iServletResponse) {

        final AdminSession adminSession = AdminSession
                .getAdminSession(iServletRequest.getSession());
        String serverName = iServletRequest.getParameter("server");
        String interfaceName = iServletRequest.getParameter("interface");
        String serviceName = iServletRequest.getParameter("service");
        String operationName = iServletRequest.getParameter("operation");
        String endpointName = iServletRequest.getParameter("endpoint");
        String mep = iServletRequest.getParameter("mep");
        String content = iServletRequest.getParameter("content");
        String attachmentFiles = iServletRequest.getParameter("server");
        String properties = iServletRequest.getParameter("server");
        String subject = iServletRequest.getParameter("server");
        /** set navigation info* */
        adminSession.setSelectedMenu(AdminSession.PETALS_ADMINISTRATION);

        try {
            final CollectorServer collector = CollectorManager.getCollector();
            CollectorClient cc = collector.getCollectorClient(serverName);
            ChannelClientJMXClient channelClient = cc.getChannelClient();
            ChannelClientListener listener = new ChannelClientListener();
            Map<String, Object> result = channelClient.getChannelClientServiceClient().sendSync(
                    interfaceName, serviceName, operationName, endpointName, mep, content, null,
                    null, null);

            String response = (String) result.get(ChannelClientReportKeyNames.CONTENT);
            /** Write the text to response * */
            iServletResponse.setContentType("text/html");
            PrintWriter out = iServletResponse.getWriter();
            out.println(response);
            out.flush();

        } catch (final DataCollectorClientException e) {
            this.fillInfoBeanWithExceptionMessage(adminSession, SendMessage.actionTitle, e
                    .getMessage());
            this.log.error("Error : can't " + SendMessage.actionTitle, e);
        } catch (final DataCollectorServerException e) {
            this.fillInfoBeanWithExceptionMessage(adminSession, SendMessage.actionTitle, e
                    .getMessage());
            this.log.error("Error : can't " + SendMessage.actionTitle, e);
        } catch (DataCollectorClientDoesNotExistException e) {
            this.fillInfoBean(adminSession,
                    "Can't reach PEtALS client, your server list must be out of date",
                    InfoBean.ERROR_MESSAGE);
            this.log.error("Error : can't " + actionTitle, e);
        } catch (DataCollectorServerNotInitializedException e) {
            CollectorManager.resetCollector();
            this.fillInfoBean(adminSession, "Connection to PEtALS lost, please refresh",
                    InfoBean.ERROR_MESSAGE);
            this.log.error("Connection to PEtALS lost : can't " + actionTitle, e);
            return LogonUtil.getLogonForward(iServletRequest);
        } catch (PerformActionErrorException e) {
            this.fillInfoBean(adminSession, "Can't perform action in PEtALS Channel Client",
                    InfoBean.ERROR_MESSAGE);
            this.log.error("Error : can't " + SendMessage.actionTitle, e);
        } catch (ConnectionErrorException e) {
            this.fillInfoBean(adminSession, "Can't perform action in PEtALS Channel Client",
                    InfoBean.ERROR_MESSAGE);
            this.log.error("Error : can't " + SendMessage.actionTitle, e);
        } catch (ChannelClientDoesNotExistException e) {
            this.fillInfoBean(adminSession, "Can't perform action in PEtALS Channel Client",
                    InfoBean.ERROR_MESSAGE);
            this.log.error("Error : can't " + SendMessage.actionTitle, e);
        } catch (ChannelClientServiceErrorException e) {
            this.fillInfoBean(adminSession, "Can't perform action in PEtALS Channel Client",
                    InfoBean.ERROR_MESSAGE);
            this.log.error("Error : can't " + SendMessage.actionTitle, e);
        } catch (IOException e) {
            this.fillInfoBean(adminSession, "Can't perform action in PEtALS Channel Client",
                    InfoBean.ERROR_MESSAGE);
            this.log.error("Error : can't " + SendMessage.actionTitle, e);
        }
        finally{
            return null;
        }
        // return iActionMapping.findForward("success");
    }

}
