/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.ui.monitoring;

import java.util.ArrayList;
import java.util.Date;
import java.util.Map;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.DynaActionForm;
import org.ow2.petals.tools.webadmin.cron.CollectorManager;
import org.ow2.petals.tools.webadmin.datacollector.client.exception.DataCollectorClientException;
import org.ow2.petals.tools.webadmin.datacollector.server.CollectorServer;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorClientDoesNotExistException;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerException;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerNotInitializedException;
import org.ow2.petals.tools.webadmin.ui.infra.action.MonitoringAction;
import org.ow2.petals.tools.webadmin.ui.infra.bean.ComponentLVO;
import org.ow2.petals.tools.webadmin.ui.infra.bean.InfoBean;
import org.ow2.petals.tools.webadmin.ui.infra.bean.QueueLVO;
import org.ow2.petals.tools.webadmin.ui.infra.session.AdminSession;
import org.ow2.petals.tools.webadmin.ui.logon.LogonUtil;
import org.ow2.petals.tools.webadmin.util.DateHelper;
import org.ow2.petals.tools.webadmin.util.StringHelper;

/**
 * @author mcarpentier - EBM Websourcing
 */
public class ShowQueuesStatus extends MonitoringAction {

    private static final String actionTitle = "get PEtALS queues current status";

    final Logger log = Logger.getLogger(ShowQueuesStatus.class);

    @Override
    public ActionForward executeMonitoringAuthentified(ActionMapping iActionMapping,
            ActionForm iActionForm, HttpServletRequest iServletRequest,
            HttpServletResponse iServletResponse) {

        final AdminSession adminSession = AdminSession
                .getAdminSession(iServletRequest.getSession());

        /** set navigation info* */
        adminSession.setSelectedMenu(AdminSession.MONITORING);

        String serverName = iServletRequest.getParameter("server");

        final DynaActionForm form = (DynaActionForm) iActionForm;

        if (StringHelper.isNullOrEmpty(serverName)) {
            serverName = form.getString("serverName");
        }
        final ArrayList<ComponentLVO> componentsLVO = new ArrayList<ComponentLVO>();
        try {
            final CollectorServer collector = CollectorManager.getCollector();
            final Map<String, Map<String, Long>> allQueues = collector.getCollectorClient(
                    serverName).getTransportMonitoringService().getAllCurrentComponentQueuesStatus(
                    serverName);

            for (final String component : allQueues.keySet()) {
                final ComponentLVO cLVO = new ComponentLVO(component, "unknown");
                final Map<String, Long> queues = allQueues.get(component);
                for (final String queue : queues.keySet()) {
                    final String nameArray[] = queue.split("-");
                    String formattedQueue = "";
                    if (nameArray.length > 1) {
                        formattedQueue = nameArray[nameArray.length - 1] + " queue";
                    } else {
                        formattedQueue = queue;
                    }
                    final QueueLVO qLVO = new QueueLVO(formattedQueue, "unknown", String
                            .valueOf(queues.get(queue)));
                    cLVO.addQueue(qLVO);
                }
                componentsLVO.add(cLVO);
            }
            form.set("components", componentsLVO);
            form.set("time", DateHelper.format(new Date()));

            form.set("serverName", serverName);
        } catch (final DataCollectorServerException e) {
            this.fillInfoBeanWithExceptionMessage(adminSession, ShowQueuesStatus.actionTitle, e
                    .getMessage());
            this.log.error("Error : can't " + ShowQueuesStatus.actionTitle, e);
        } catch (final DataCollectorClientException e) {
            this.fillInfoBeanWithExceptionMessage(adminSession, ShowQueuesStatus.actionTitle, e
                    .getMessage());
            this.log.error("Error : can't " + ShowQueuesStatus.actionTitle, e);
        } catch (DataCollectorClientDoesNotExistException e) {
            this.fillInfoBean(adminSession,
                    "Can't reach PEtALS client, your server list must be out of date",
                    InfoBean.ERROR_MESSAGE);
            this.log.error("Error : can't " + actionTitle, e);
        } catch (DataCollectorServerNotInitializedException e) {
            CollectorManager.resetCollector();
            this.fillInfoBean(adminSession, "Connection to PEtALS lost, please refresh",
                    InfoBean.ERROR_MESSAGE);
            this.log.error("Connection to PEtALS lost : can't " + actionTitle, e);
            return LogonUtil.getLogonForward(iServletRequest);
        }
        return iActionMapping.findForward("success");
    }
}
