/**
 * PETALS - PETALS Services Platform.
 * Copyright (c) 2007 EBM Websourcing, http://www.ebmwebsourcing.com/
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.ui.portlet;

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.action.DynaActionForm;
import org.ow2.petals.tools.webadmin.cron.CollectorManager;
import org.ow2.petals.tools.webadmin.datacollector.client.exception.DataCollectorClientException;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorClientDoesNotExistException;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerException;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerNotInitializedException;
import org.ow2.petals.tools.webadmin.ui.infra.action.AuthentifiedAction;
import org.ow2.petals.tools.webadmin.ui.infra.bean.DomainLVO;
import org.ow2.petals.tools.webadmin.ui.infra.bean.InfoBean;
import org.ow2.petals.tools.webadmin.ui.infra.session.AdminSession;
import org.ow2.petals.tools.webadmin.ui.logon.LogonUtil;
import org.ow2.petals.tools.webadmin.util.DataCollectorHelper;
import org.ow2.petals.tools.webadmin.util.StringHelper;

/**
 * Upload a component to a temporary directory and install it
 * 
 * request params : componentFile, server
 * 
 * @author mcarpentier - EBM Websourcing
 * 
 */
public class ShowPortlet extends AuthentifiedAction {

    final Logger log = Logger.getLogger(ShowPortlet.class);

    private static final String actionTitle = "get PEtALS components";

    @Override
    public ActionForward executeAuthentified(ActionMapping iActionMapping, ActionForm iActionForm,
            HttpServletRequest iServletRequest, HttpServletResponse iServletResponse) {

        DynaActionForm form = (DynaActionForm) iActionForm;
        AdminSession adminSession = AdminSession.getAdminSession(iServletRequest.getSession());

        /** set navigation info* */
        adminSession.setSelectedMenu(AdminSession.JUDDI_ADMINISTRATION);

        /** get all form parameters* */
        String server = form.getString("server");
        String component = form.getString("component");
        String viewAction = form.getString("viewAction");

        /** refresh action datas * */
        if (StringHelper.isNullOrEmpty(iServletRequest.getParameter("server"))) {
            server = iServletRequest.getParameter("server");
            form.set("server", iServletRequest.getParameter("server"));
        }
        if (StringHelper.isNullOrEmpty(iServletRequest.getParameter("component"))) {
            component = iServletRequest.getParameter("component");
            form.set("component", iServletRequest.getParameter("component"));
        }
        if (StringHelper.isNullOrEmpty(iServletRequest.getParameter("viewAction"))) {
            viewAction = iServletRequest.getParameter("viewAction");
            form.set("viewAction", iServletRequest.getParameter("viewAction"));
        }

        try {
            if (adminSession.getDomain() == null) {
                adminSession.setDomain(DataCollectorHelper.updateDomainInfo(false));
            }
            DomainLVO nLVO = adminSession.getDomain();

            /** TODO : subdomain information* */
            form.set("servers", nLVO.getSubdomains().values().iterator().next().getServersName());
            form.set("components", nLVO.getSubdomains().values().iterator().next().getServerLVO(
                    server).getComponents());
        } catch (DataCollectorServerException e) {
            this.fillInfoBeanWithExceptionMessage(adminSession, actionTitle, e.getMessage());
            this.log.error("Error : can't " + actionTitle, e);
        } catch (DataCollectorClientException e) {
            this.fillInfoBeanWithExceptionMessage(adminSession, actionTitle, e.getMessage());
            this.log.error("Error : can't " + actionTitle, e);
        } catch (DataCollectorClientDoesNotExistException e) {
            this.fillInfoBean(adminSession,
                    "Can't reach PEtALS client, your server list must be out of date",
                    InfoBean.ERROR_MESSAGE);
            this.log.error("Error : can't " + actionTitle, e);
        } catch (DataCollectorServerNotInitializedException e) {
            CollectorManager.resetCollector();
            this.fillInfoBean(adminSession, "Connection to PEtALS lost, please refresh",
                    InfoBean.ERROR_MESSAGE);
            this.log.error("Connection to PEtALS lost : can't " + actionTitle, e);
            return LogonUtil.getLogonForward(iServletRequest);
        }

        return iActionMapping.findForward("success");

    }
}
