/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.tools.webadmin.ui.proxyservice;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.util.Iterator;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.servlet.ServletContext;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;

import org.apache.log4j.Logger;
import org.apache.struts.action.ActionForm;
import org.apache.struts.action.ActionForward;
import org.apache.struts.action.ActionMapping;
import org.apache.struts.validator.DynaValidatorForm;
import org.ow2.easywsdl.wsdl.api.Endpoint;
import org.ow2.easywsdl.wsdl.api.Service;
import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.petals.tools.webadmin.ui.infra.action.AuthentifiedAction;
import org.ow2.petals.tools.webadmin.ui.infra.session.AdminSession;
import org.ow2.petals.tools.webadmin.util.FilePermissionHelper;

/**
 * @author aruffie
 * 
 */
public class AddInternalServiceWsdl extends AuthentifiedAction {

    final Logger log = Logger.getLogger(AddInternalServiceWsdl.class);

    private static final String actionTitle = "add Proxy Service from WSDL";

    @Override
    public ActionForward executeAuthentified(ActionMapping iActionMapping, ActionForm iActionForm,
            HttpServletRequest iServletRequest, HttpServletResponse iServletResponse) {

        AdminSession adminSession = AdminSession.getAdminSession(iServletRequest.getSession());
        DynaValidatorForm dynaValidatorForm = (DynaValidatorForm) iActionForm;

        /** set navigation info* */
        adminSession.setSelectedMenu(AdminSession.PROXY_SERVICE);

        /*
         * recover all parameters provide in the request when, user click on
         * element of the list
         */

        String endpointName = iServletRequest.getParameter("endpointName");
        String serviceName = iServletRequest.getParameter("serviceName");
        String wsdlFile = iServletRequest.getParameter("wsdlFile");
        File file;
        try {
            file = new File(checkRepository(iServletRequest, iServletResponse) + "/" + wsdlFile);
        } catch (IOException e1) {
            iServletRequest.setAttribute("error", e1.getMessage());
            e1.printStackTrace();
            return iActionMapping.findForward("error");
        } catch (RuntimeException e1) {
            iServletRequest.setAttribute("wsdlFile", iServletRequest.getSession().getAttribute(
                    "wsdlFile"));
            iServletRequest.setAttribute("error", e1.getMessage());
            e1.printStackTrace();
            return iActionMapping.findForward("error");
        }

        List<Service> servicesList = null;
        try {
            // parse wsdl file with easy wsdl to find all services
            servicesList = HandlerAddWsdl.getServices(file);
        } catch (FileNotFoundException e) {
            System.out.println("Exception occured for get the wsdl file: " + e.getMessage());
        }

        Iterator serviceIt = servicesList.iterator();
        /*
         * now iterate on all services in the list, to find the service/endpoint
         * selected in the menu
         */
        try {
            while (serviceIt.hasNext()) {
                Service service = (Service) serviceIt.next();

                /*
                 * Sauvegarde du service en session pour y avoir accés plustard,
                 * lors de la validation du proxy service par l'utilisateur
                 */

                iServletRequest.getSession().setAttribute("serviceName",
                        service.getInterface().getQName().toString());

                List<Endpoint> endpointsList = service.getEndpoints();
                Iterator endpointIt = endpointsList.iterator();
                while (endpointIt.hasNext()) {
                    Endpoint endpoint = (Endpoint) endpointIt.next();

                    if (endpoint.getService().getQName().toString().equals(
                            serviceName)
                            && (endpoint.getName().toString().equals(endpointName))) {

                        /*
                         * Sauvegarde du endpoint/service en session pour y
                         * avoir accés plustard, lors de la validation du proxy
                         * service par l'utilisateur
                         */
                        iServletRequest.getSession().setAttribute("serviceName",
                                service.getQName().toString());
                        iServletRequest.getSession().setAttribute("endpointName",
                                endpoint.getName().toString());

                        /*
                         * Add the defaults values of the form display in the
                         * frame
                         */
                        dynaValidatorForm.set("proxyName", endpoint.getName() + "ProxySA");
                        dynaValidatorForm.set("wsdl", wsdlFile);
                        dynaValidatorForm.set("consumeSUName", endpoint.getName()
                                + "SOAPProxySUConsume");
                        dynaValidatorForm.set("consumeURI", endpoint.getName()
                                + "SOAPProxy");
                        dynaValidatorForm.set("restconsumeSUName", endpoint.getName()
                                + "RESTProxySUConsume");
                        dynaValidatorForm.set("restconsumeURI", endpoint.getName()
                                + "RESTProxy");
                        dynaValidatorForm.set("timeout", "30");
                        
                        /*
                         * define if is a soap or rest type with regex
                         */
                        Pattern pattern = Pattern.compile("soap");
                        Matcher matcher = pattern.matcher(endpoint.getBinding()
                                .getTransportProtocol());

                        if (matcher.find()) {
                            dynaValidatorForm.set("soap", true);
                            dynaValidatorForm.set("rest", false);
                        } else {
                            dynaValidatorForm.set("soap", false);
                            dynaValidatorForm.set("rest", true);
                        }
                    }
                }
            }

        } catch (WSDLException e) {
            // TODO Auto-generated catch block
            System.out.println("Error occured in AddServiceWsdl: " + e.getMessage());
        }

        // save all services to display these in the left menu
        try {
            iServletRequest.setAttribute("wsdlServices", HandlerAddWsdl.getServices(file));
        } catch (FileNotFoundException e) {
            iServletRequest.setAttribute("errorWsdlFile", true);
            System.out.println("Exception occured for get the wsdl file: " + e.getMessage());
            return iActionMapping.findForward("error");
        }

        iServletRequest.getSession().setAttribute("wsdlFile", wsdlFile);
        return iActionMapping.findForward("success");

    }

    private String checkRepository(HttpServletRequest iServletRequest,
            HttpServletResponse iServletResponse) throws IOException {
        // create repository and upload file
        final ServletContext context = iServletRequest.getSession().getServletContext();
        String reqPath = iServletRequest.getContextPath();
        reqPath = reqPath.substring(reqPath.lastIndexOf("/") + 1);
        final String realContextPath = context.getRealPath(iServletRequest.getContextPath());
        final String rootPath = realContextPath.substring(0, realContextPath.lastIndexOf(reqPath));
        File uploadDir = new File(rootPath + "upload/");

        if (!(uploadDir.exists())) {
            uploadDir.mkdir();
        }
        try {
            FilePermissionHelper.setRights(uploadDir, new int[] { 7, 7, 7 }, false);
        } catch (IOException e1) {
            e1.printStackTrace();
            throw new RuntimeException(e1.getMessage());
        }
        return uploadDir.getCanonicalPath();
    }
}
