/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.tools.webadmin.ui.proxyservice;

import java.io.File;
import java.io.IOException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.URL;

import javax.servlet.http.HttpServletRequest;

import org.ow2.petals.tools.webadmin.cron.CollectorManager;
import org.ow2.petals.tools.webadmin.datacollector.client.exception.DataCollectorClientException;
import org.ow2.petals.tools.webadmin.datacollector.server.CollectorServer;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorClientDoesNotExistException;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerException;
import org.ow2.petals.tools.webadmin.datacollector.server.exception.DataCollectorServerNotInitializedException;
import org.ow2.petals.tools.webadmin.util.ArtifactInstallationHelper;

/**
 * @author aruffie
 * 
 */
public class ArtifactDeployer {

    public static String saDeployement(HttpServletRequest iServletRequest, File saFile,
            String serverName, String subdomain) {
        // check if we use http or ftp download
        URI artifactServerURI = null;
        try {
            artifactServerURI = ArtifactInstallationHelper.getURI(iServletRequest);
        } catch (final URISyntaxException e) {
            System.out.println("Exception occured durin getURI() in SA Deployement");
        }
        if ("ftp".equalsIgnoreCase(artifactServerURI.getScheme())) {
            return "Error: FTP artifact installation not supported yet";

        } else if ("http".equalsIgnoreCase(artifactServerURI.getScheme())) {
            try {
                if (serverName != null) {
                    final CollectorServer collector = CollectorManager.getCollector();

                    // constructs http url of file uploaded
                    final int lastSeparator = artifactServerURI.toString().lastIndexOf("/") + 1;
                    URL httpUrl = new URL(artifactServerURI.toString().substring(0, lastSeparator)
                            + "upload/" + saFile.getName());
                    collector.getCollectorClient(serverName).getDeployService()
                            .deployServiceAssembly(httpUrl);
                }
            } catch (final IOException e) {
                System.out.println("Error: " + e.getMessage());
                throw new RuntimeException("Error : SA archive file not readable - "
                        + e.getMessage());
            } catch (final DataCollectorClientException e) {
                System.out.println("Error: " + e.getMessage());
                throw new RuntimeException("Error :" + saFile.getName() + e.getMessage());
            } catch (final DataCollectorServerException e) {
                System.out.println("Error: " + e.getMessage());
                throw new RuntimeException("Error :" + saFile.getName() + e.getMessage());
            } catch (DataCollectorClientDoesNotExistException e) {
                System.out.println("Error: " + e.getMessage());
                throw new RuntimeException(
                        "Error: Can't reach PEtALS client, your server list must be out of date - "
                                + e.getMessage());
            } catch (final DataCollectorServerNotInitializedException e) {
                CollectorManager.resetCollector();
                System.out.println("Error: Connection to PEtALS lost, please refresh");
                throw new RuntimeException("Error: Connection to PEtALS lost, please refresh - "
                        + e.getMessage());
            }

        }
        return null;
    }

}
