/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.tools.webadmin.ui.proxyservice;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileWriter;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.ow2.easywsdl.wsdl.WSDLFactory;
import org.ow2.easywsdl.wsdl.api.Description;
import org.ow2.easywsdl.wsdl.api.Endpoint;
import org.ow2.easywsdl.wsdl.api.Service;
import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.easywsdl.wsdl.api.WSDLReader;
import org.ow2.easywsdl.wsdl.api.abstractItf.AbsItfDescription.WSDLVersionConstants;
import org.ow2.petals.tools.webadmin.util.FilePermissionHelper;

import com.ebmwebsourcing.commons.jbi.sugenerator.beans.SaBean;
import com.ebmwebsourcing.commons.jbi.sugenerator.beans.SuBean;
import com.ebmwebsourcing.commons.jbi.sugenerator.beans.SuBeanForSa;
import com.ebmwebsourcing.commons.jbi.sugenerator.beans.XmlElement;
import com.ebmwebsourcing.commons.jbi.sugenerator.utils.JbiXmlGenerator;
import com.ebmwebsourcing.commons.jbi.sugenerator.utils.JbiZipper;

/**
 * @author aruffie
 *
 */
public class HandlerInternalProxyService {

    // Component used by the proxy SA
    private static final String COMPONENTNAME = new String("petals-bc-soap");
    private static final String RESTMODE = new String("REST");
    private File repository = null;
    private String serviceName = null;
    
    //Addresse ou le service sera consommé 
    private String consumeURI = null;
    private String consumeSUName = null;
    private String interfaceNameProvider = null;
    private String interfaceNameNsUrlProvider = null;
    private String addrSUProvide = null;
    private String timeout = null;
    private String proxyName = null;
    private File wsdlFile = null;
    private String endpointName = null;
    private String wsdlVersion = null;
    private boolean rest = false;
    private boolean soap = true;
    private String restConsumeSUName = null;
    private String restConsumeURI = null;
    private String restNSPrefix = null;
    private String restNSURI = null;
    private String restRMPrefixResponse = null;
    /**
     * Function allow you to create the proxy with informations content in the
     * handler, and return the SA zip file
     * 
     * @return
     * @throws IOException
     */
    public File addProxy() throws IOException {


        final List<SuBean> suBeans = new ArrayList<SuBean>();



        // Add that SU in the su list of the SA
        if(this.soap){
            final SuBean suSoapConsume = this.createSOAPConsumeSU();
            suBeans.add(suSoapConsume);
        }
        //If rest
        if(this.rest){
            final SuBean suRestConsume = this.createRestConsumeSU();
            suBeans.add(suRestConsume);
        }
        // Generate a jbi.xml file for a service assembly.
        SaBean saBean = new SaBean();
        saBean.setSaName(proxyName);
        saBean.setDescription("SA generated for the " + proxyName);
        List<SuBeanForSa> sus = new ArrayList<SuBeanForSa>();
        List<File> suZipFiles = new ArrayList<File>();
        File saZipFile = null;
        for(SuBean suBean : suBeans){
            // Generate a jbi.xml file a service unit.
            String jbiXmlForSu = JbiXmlGenerator.getInstance().generateJbiXmlFileForSu(suBean);

            File tempSaDirectory = new File(repository.getCanonicalPath());
            if (!tempSaDirectory.exists()) {
                tempSaDirectory.mkdir();
            }
            try {
                FilePermissionHelper.setRights(tempSaDirectory, new int[] { 7, 7, 7 }, false);
            } catch (final IOException e1) {
                e1.printStackTrace();
                throw new RuntimeException(e1.getMessage());
            }

            File jbiSu = new File(repository.getCanonicalPath() + "/jbi.xml");
            File suFile = null;
            try {
                FileWriter writer = new FileWriter(jbiSu);
                writer.write(jbiXmlForSu);
                writer.close();
                List<File> rootFiles = new ArrayList<File>();

                JbiZipper jbiZipper = JbiZipper.getInstance();
                // Save SU in the repository
                try {
                      rootFiles.add(jbiSu);
                      for(final XmlElement specificElement : suBean.specificElements){
                          if((specificElement.getName().equals("soap:mode"))&&(specificElement.getValue().equals(RESTMODE))){
                              suFile = new File(repository.getCanonicalPath() + "/" + restConsumeSUName
                                      + ".zip");
                          } else {
                              if((specificElement.getName().equals("soap:address"))&&(specificElement.getValue().equals(this.consumeURI))){
                                  suFile = new File(repository.getCanonicalPath() + "/" + consumeSUName
                                          + ".zip");
                              }
                          }
                      }
                    suZipFiles.add(jbiZipper.createSuZipFile(suFile, rootFiles));

                } catch (FileNotFoundException e) {
                    System.out.println("FilenotFoundException in suFile initialization: "
                            + e.getMessage());
                    e.printStackTrace();
                    throw new RuntimeException(e.getMessage());
                } catch (Exception e) {
                    System.out.println("Exception in suFile initialization: " + e.getMessage());
                    e.printStackTrace();
                    throw new RuntimeException(e.getMessage());
                }

            } catch (IOException e) {
                System.out.println("IOException occured when " + jbiSu.getName() + " was wrote");
                e.printStackTrace();
            }

            SuBeanForSa suBeanForSa = new SuBeanForSa(suBean);
            suBeanForSa.setZipArtifact(suFile.getName());
    
            for(final XmlElement specificElement : suBean.specificElements){
                if((specificElement.getName().equals("soap:mode"))&&(specificElement.getValue().equals(RESTMODE))){
                    suBeanForSa.setSuName(restConsumeSUName);
                } else {
                    if((specificElement.getName().equals("soap:address"))&&(specificElement.getValue().equals(this.consumeURI))){
                        suBeanForSa.setSuName(consumeSUName);
                    }
                }
            }
            suBeanForSa.setComponentName(COMPONENTNAME);

            sus.add(suBeanForSa);

            jbiSu.delete();

            saBean.setSus(sus);

            // Generate jbi file for the SA
            String jbiXmlForSa = JbiXmlGenerator.getInstance().generateJbiXmlFileForSa(saBean);
            File jbiSa = new File(repository.getCanonicalPath() + "/jbi.xml");
            // Création des zip de la SA
            try {
                FileWriter writer = new FileWriter(jbiSa);
                writer.write(jbiXmlForSa);
                writer.close();
                JbiZipper jbiZipper = JbiZipper.getInstance();
    
                // Save the SA zip file in the repository
                File saFile = new File(repository.getCanonicalPath() + "/" + proxyName + ".zip");
                saZipFile = jbiZipper.createSaZipFile(saFile, suZipFiles, jbiSa);
            } catch (IOException e) {
                System.out.println("IOException occured when " + jbiSa.getName() + " was wrote");
                e.printStackTrace();
                throw new RuntimeException(e.getMessage());
            }
            jbiSa.delete();
        }
        return saZipFile;
    }

    /**
     * Function allow to get Description from a wsdl file
     * 
     * @return Description
     * @throws WSDLException
     * @throws URISyntaxException
     * @throws MalformedURLException
     * @throws PEtALSCDKException
     */
    public Description readWSDL() throws WSDLException, URISyntaxException, MalformedURLException {
        WSDLReader wsdlReader = WSDLFactory.newInstance().newWSDLReader();
        return wsdlReader.readWSDL(new URI(wsdlFile.toURI().toString()));
    }
    
    /**
     * Function allow you to read description
     * 
     * @param description
     */
    public void readDescription(Description description) {
        // Récupération des différentes services
        List servicesList = description.getServices();
        Iterator serviceIt = servicesList.iterator();
        while (serviceIt.hasNext()) {
            Service s = (Service) serviceIt.next();

            List<Endpoint> endpointsList = s.getEndpoints();
            Iterator endpointIt = endpointsList.iterator();
            while (endpointIt.hasNext()) {
                Endpoint e = (Endpoint) endpointIt.next();

                if ((s.getQName().toString().equals(serviceName))
                        && (e.getName().toString().equals(endpointName))) {
                    addrSUProvide = endpointName;
                    interfaceNameProvider = e.getBinding().getInterface().getQName().getLocalPart()
                            .toString();
                    interfaceNameNsUrlProvider = e.getBinding().getInterface().getQName()
                            .getNamespaceURI();
                    if (description.getVersion().equals(WSDLVersionConstants.WSDL11)) {
                        wsdlVersion = "1.1";
                    } else {
                        if (description.getVersion().equals(WSDLVersionConstants.WSDL20)) {
                            wsdlVersion = "2.0";
                        }
                    }

                }
            }

        }
    }

    public HandlerInternalProxyService(String endpointName, String serviceName,
            File wsdlFile, String timeout, String proxyName, String consumeSUName,
            String consumeURI,File repository ,boolean soap, boolean rest,
            String restConsumeSUName,String restConsumeURI,String restNSPrefix,
            String restNSURI, String restRMPrefixResponse) {
        super();
        this.endpointName = endpointName;
        this.serviceName = serviceName;
        this.wsdlFile = wsdlFile;
        this.timeout = timeout;
        this.proxyName = proxyName;
        this.consumeSUName = consumeSUName;
        this.consumeURI = consumeURI;
        this.repository = repository;
        this.soap = soap;
        this.rest = rest;
        this.restConsumeSUName = restConsumeSUName;
        this.restConsumeURI = restConsumeURI;
        this.restNSPrefix = restNSPrefix;
        this.restNSURI = restNSURI;
        this.restRMPrefixResponse = restRMPrefixResponse;
    }
    
   
    private SuBean createRestConsumeSU(){
        
        SuBean suBean = null;
        
        final ArrayList<XmlElement> componentElementsConsumer = new ArrayList<XmlElement>();

        /*
         * Cut the LocalName and Namespace form the QName to avoid duplication
         * in jbi consume service-name
         */
   
        final String LocalName = serviceName
                .substring(serviceName.indexOf("}") + 1, serviceName.length());

        XmlElement element = new XmlElement();
        element.setName("soap:address");
        element.setValue(restConsumeURI);
        componentElementsConsumer.add(element);
        
        element = new XmlElement();
        element.setName("soap:mode");
        element.setValue(RESTMODE);
        componentElementsConsumer.add(element);
        
        if(this.restNSPrefix != null){
            element = new XmlElement();
            element.setName("soap:rest-add-namespace-prefix");
            element.setValue(this.restNSPrefix);
            componentElementsConsumer.add(element);
        }
        if(this.restNSURI != null){
            element = new XmlElement();
            element.setName("soap:rest-add-namespace-uri");
            element.setValue(this.restNSURI);
            componentElementsConsumer.add(element);
        }
        if(this.restRMPrefixResponse != null){
            element = new XmlElement();
            element.setName("soap:rest-remove-prefix-on-response");
            element.setValue(this.restRMPrefixResponse);
            componentElementsConsumer.add(element);
        }
        // Create the Consumer SU
        suBean = SuCreator.createSuBeanConsume(interfaceNameProvider,
                interfaceNameNsUrlProvider, LocalName, interfaceNameNsUrlProvider, addrSUProvide,
                "whatever", consumeURI, consumeURI, consumeURI, "ns", "operation",
                timeout, componentElementsConsumer);
        return suBean;
    }
    
    private SuBean createSOAPConsumeSU(){
        final ArrayList<XmlElement> componentElementsConsumer = new ArrayList<XmlElement>();
        /*
         * Cut the LocalName and Namespace form the QName to avoid duplication
         * in jbi consume service-name
         */
   
        final String LocalName = serviceName
                .substring(serviceName.indexOf("}") + 1, serviceName.length());

        XmlElement element = new XmlElement();
        element.setName("soap:address");
        element.setValue(consumeURI);
        componentElementsConsumer.add(element);

        // Create the Consumer SU
        SuBean suConsume = SuCreator.createSuBeanConsume(interfaceNameProvider,
                interfaceNameNsUrlProvider, LocalName, interfaceNameNsUrlProvider, addrSUProvide,
                "whatever", consumeURI, consumeURI, consumeURI, "ns", "operation",
                timeout, componentElementsConsumer);
        return suConsume;
    }
}
