/**
 * PETALS - PETALS Services Platform. Copyright (c) 2007 EBM Websourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 */
package org.ow2.petals.tools.webadmin.ui.proxyservice;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.net.InetAddress;
import java.net.MalformedURLException;
import java.net.URI;
import java.net.URISyntaxException;
import java.net.UnknownHostException;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.ow2.easywsdl.wsdl.WSDLFactory;
import org.ow2.easywsdl.wsdl.api.Description;
import org.ow2.easywsdl.wsdl.api.Endpoint;
import org.ow2.easywsdl.wsdl.api.Service;
import org.ow2.easywsdl.wsdl.api.WSDLException;
import org.ow2.easywsdl.wsdl.api.WSDLReader;
import org.ow2.easywsdl.wsdl.api.WSDLWriter;
import org.ow2.easywsdl.wsdl.api.abstractItf.AbsItfDescription.WSDLVersionConstants;
import org.ow2.petals.tools.webadmin.util.FilePermissionHelper;
import org.w3c.dom.Document;

import com.ebmwebsourcing.commons.jbi.sugenerator.beans.SaBean;
import com.ebmwebsourcing.commons.jbi.sugenerator.beans.SuBean;
import com.ebmwebsourcing.commons.jbi.sugenerator.beans.SuBeanForSa;
import com.ebmwebsourcing.commons.jbi.sugenerator.beans.XmlElement;
import com.ebmwebsourcing.commons.jbi.sugenerator.utils.JbiXmlGenerator;
import com.ebmwebsourcing.commons.jbi.sugenerator.utils.JbiZipper;

/**
 * @author aruffie
 * 
 */
public class HandlerProxyService {

    // Component used by the proxy SA
    private static final String COMPONENTNAME = new String("petals-bc-soap");
    private static final String RESTMODE = new String("REST");
    private static final String SOAPMODE = new String("SOAP");
    private File repository = null;

    private File wsdlFile = null;
    private File wsdlProvideFile = null;
    private String proxyName = null;

    private String wsdl = null;

    private String timeout = null;

    private String addrSUProvide = null;

    private String consumeSUName = null;
    private String consumeURI = null;

    private Boolean soap = null;

    private Boolean rest = null;

    private String endpointName = null;

    private String serviceName = null;

    // Begin Provider attributs
    private String interfaceNameProvider = null;

    private String interfaceNameNsUrlProvider = null;

    private String endpointNameProvider = null;

    private String namespaceProvider = null;

    private String soapAddr = null;

    private String wsdlVersion = null;

    private String restConsumeSUName = null;
    private String restConsumeURI = null;
    private String restNSPrefix= null;
    private String restNSURI = null;
    private String restRMPrefixResponse = null;
    // End Provider attributs
    public File getWsdlFile() {
        return wsdlFile;
    }

    public String getEndpointNameProvider() {
        return endpointNameProvider;
    }

    public void setEndpointNameProvider(String endpointNameProvider) {
        this.endpointNameProvider = endpointNameProvider;
    }

    public String getNamespaceProvider() {
        return namespaceProvider;
    }

    public void setNamespaceProvider(String namespaceProvider) {
        this.namespaceProvider = namespaceProvider;
    }

    public void setWsdlFile(File wsdlFile) {
        this.wsdlFile = wsdlFile;
    }

    public HandlerProxyService(File wsdlFile) {
        super();
        this.wsdlFile = wsdlFile;
    }

    public String getProxyName() {
        return proxyName;
    }

    public void setProxyName(String proxyName) {
        this.proxyName = proxyName;
    }

    public String getWsdl() {
        return wsdl;
    }

    public void setWsdl(String wsdl) {
        this.wsdl = wsdl;
    }

    public String getTimeout() {
        return timeout;
    }

    public void setTimeout(String timeout) {
        this.timeout = timeout;
    }

    public String getAddrSUProvide() {
        return addrSUProvide;
    }

    public void setAddrSUProvide(String addrSUProvide) {
        this.addrSUProvide = addrSUProvide;
    }

    public Boolean getSoap() {
        return soap;
    }

    public void setSoap(Boolean soap) {
        this.soap = soap;
    }

    public Boolean getRest() {
        return rest;
    }

    public void setRest(Boolean rest) {
        this.rest = rest;
    }

    public String getEndpointName() {
        return endpointName;
    }

    public void setEndpointName(String endpointName) {
        this.endpointName = endpointName;
    }

    public String getServiceName() {
        return serviceName;
    }

    public void setServiceName(String serviceName) {
        this.serviceName = serviceName;
    }

    public File getRepository() {
        return repository;
    }

    public void setRepository(File repository) {
        this.repository = repository;
    }

    /**
     * Function allow to get Description from a wsdl file
     * 
     * @return Description
     * @throws WSDLException
     * @throws URISyntaxException
     * @throws MalformedURLException
     * @throws PEtALSCDKException
     */
    public Description readWSDL() throws WSDLException, URISyntaxException, MalformedURLException {
        WSDLReader wsdlReader = WSDLFactory.newInstance().newWSDLReader();
        return wsdlReader.readWSDL(new URI(wsdlFile.toURI().toString()));
    }

    /**
     * Function allow you to read description
     * 
     * @param description
     */
    public void readDescription(Description description) {
        // Récupération des différentes services
        List servicesList = description.getServices();
        Iterator serviceIt = servicesList.iterator();
        namespaceProvider = description.getTargetNamespace();
        while (serviceIt.hasNext()) {
            Service s = (Service) serviceIt.next();

            List<Endpoint> endpointsList = s.getEndpoints();
            Iterator endpointIt = endpointsList.iterator();
            while (endpointIt.hasNext()) {
                Endpoint e = (Endpoint) endpointIt.next();

                if ((s.getQName().toString().equals(serviceName))
                        && (e.getName().toString().equals(endpointName))) {
                    endpointNameProvider = endpointName;
                    interfaceNameProvider = e.getBinding().getInterface().getQName().getLocalPart()
                            .toString();
                    interfaceNameNsUrlProvider = e.getBinding().getInterface().getQName()
                            .getNamespaceURI();
                    soapAddr = e.getAddress();
                    if (description.getVersion().equals(WSDLVersionConstants.WSDL11)) {
                        wsdlVersion = "1.1";
                    } else {
                        if (description.getVersion().equals(WSDLVersionConstants.WSDL20)) {
                            wsdlVersion = "2.0";
                        }
                    }

                }
            }

        }
    }

    public HandlerProxyService(String consumeSUName,String consumeURI, String addrSUProvide, String proxyName,
            Boolean rest, Boolean soap, String timeout, String wsdl, File wsdlFile) {
        super();
        this.consumeSUName = consumeSUName;
        this.consumeURI = consumeURI;
        this.addrSUProvide = addrSUProvide;
        this.proxyName = proxyName;
        this.rest = rest;
        this.soap = soap;
        this.timeout = timeout;
        this.wsdl = wsdl;
        this.wsdlFile = wsdlFile;

    }

    /**
     * Function allow you to create the proxy with informations content in the
     * handler, and return the SA zip file
     * 
     * @return
     * @throws IOException
     */
    public File addProxy() throws IOException {

        List<SuBean> suBeans = new ArrayList<SuBean>();

        // Add these SU in the su list of the SA
        suBeans.add(this.createSOAPProvideSU());
        if(this.soap){
            final SuBean suSoapConsume = this.createSOAPConsumeSU();
            suBeans.add(suSoapConsume);
        }
        //If rest
        if(this.rest){
            final SuBean suRestConsume = this.createRestConsumeSU();
            suBeans.add(suRestConsume);
        }


        // Generate a jbi.xml file for a service assembly.
        SaBean saBean = new SaBean();
        saBean.setSaName(proxyName);
        saBean.setDescription("SA generated for the " + proxyName);
        List<SuBeanForSa> sus = new ArrayList<SuBeanForSa>();
        List<File> suZipFiles = new ArrayList<File>();
        
        for (SuBean suBean : suBeans) {

            // Generate a jbi.xml file a service unit.
            String jbiXmlForSu = JbiXmlGenerator.getInstance().generateJbiXmlFileForSu(suBean);

            File tempSaDirectory = new File(repository.getCanonicalPath());
            if (!tempSaDirectory.exists()) {
                tempSaDirectory.mkdir();
            }
            try {
                FilePermissionHelper.setRights(tempSaDirectory, new int[] { 7, 7, 7 }, false);
            } catch (final IOException e1) {
                e1.printStackTrace();
                throw new RuntimeException(e1.getMessage());
            }

            File jbiSu = new File(repository.getCanonicalPath() + "/jbi.xml");
            File suFile = null;
            try {
                FileWriter writer = new FileWriter(jbiSu);
                writer.write(jbiXmlForSu);
                writer.close();
                List<File> rootFiles = new ArrayList<File>();

                JbiZipper jbiZipper = JbiZipper.getInstance();
                // Save SU in the repository
 
                try {

                    rootFiles.add(jbiSu);
                    for(final XmlElement specificElement : suBean.specificElements){
                        if((specificElement.getName().equals("soap:mode"))&&(specificElement.getValue().equals(RESTMODE))){
                            suFile = new File(repository.getCanonicalPath() + "/" + restConsumeSUName
                                    + ".zip");
                        } else {
                            if((specificElement.getName().equals("soap:address"))&&(specificElement.getValue().equals(this.consumeURI))){
                                suFile = new File(repository.getCanonicalPath() + "/" + this.consumeSUName
                                        + ".zip");
                            } else {
                                if((specificElement.getName().equals("soap:mode"))&&(specificElement.getValue().equals(SOAPMODE))){
                                    rootFiles.add(this.wsdlProvideFile);
                                    suFile = new File(repository.getCanonicalPath() + "/" + addrSUProvide
                                            + ".zip");
                                }
                            }
                        }
                    }

                    suZipFiles.add(jbiZipper.createSuZipFile(suFile, rootFiles));

                } catch (FileNotFoundException e) {
                    System.out.println("FilenotFoundException in suFile initialization: "
                            + e.getMessage());
                    e.printStackTrace();
                    throw new RuntimeException(e.getMessage());
                } catch (Exception e) {
                    System.out.println("Exception in suFile initialization: " + e.getMessage());
                    e.printStackTrace();
                    throw new RuntimeException(e.getMessage());
                }

            } catch (IOException e) {
                System.out.println("IOException occured when " + jbiSu.getName() + " was wrote");
                e.printStackTrace();
            }

            SuBeanForSa suBeanForSa = new SuBeanForSa(suBean);
            suBeanForSa.setZipArtifact(suFile.getName());

            for(final XmlElement specificElement : suBean.specificElements){
                if((specificElement.getName().equals("soap:mode"))&&(specificElement.getValue().equals(RESTMODE))){
                    suBeanForSa.setSuName(restConsumeSUName);
                } else {
                    if((specificElement.getName().equals("soap:address"))&&(specificElement.getValue().equals(this.consumeURI))){
                        suBeanForSa.setSuName(this.consumeSUName);
                    } else {
                        if((specificElement.getName().equals("soap:mode"))&&(specificElement.getValue().equals(SOAPMODE))){
                            suBeanForSa.setSuName(addrSUProvide);
                        }
                    }
                }
            }
            suBeanForSa.setComponentName(COMPONENTNAME);

            sus.add(suBeanForSa);

            jbiSu.delete();

        }

        /*
         * witch the order of the su, because the provider su need to be the
         * first in the sa jbi file
         */
        //sus = switchSu(sus);
        saBean.setSus(sus);

        // Generate jbi file for the SA
        String jbiXmlForSa = JbiXmlGenerator.getInstance().generateJbiXmlFileForSa(saBean);
        File jbiSa = new File(repository.getCanonicalPath() + "/jbi.xml");
        File saZipFile = null;
        // Création des zip de la SA
        try {
            FileWriter writer = new FileWriter(jbiSa);
            writer.write(jbiXmlForSa);
            writer.close();
            JbiZipper jbiZipper = JbiZipper.getInstance();

            // Save the SA zip file in the repository
            File saFile = new File(repository.getCanonicalPath() + "/" + proxyName + ".zip");
            saZipFile = jbiZipper.createSaZipFile(saFile, suZipFiles, jbiSa);
        } catch (IOException e) {
            System.out.println("IOException occured when " + jbiSa.getName() + " was wrote");
            e.printStackTrace();
            throw new RuntimeException(e.getMessage());
        }
        jbiSa.delete();
        return saZipFile;
    }

    public HandlerProxyService(String consumeSUName, String consumeURI, String addrSUProvide, String endpointName,
            String proxyName, Boolean rest, String serviceName, Boolean soap, String timeout,
            String wsdl, File wsdlFile,String restConsumeSUName,String restConsumeURI,
            String restNSPrefix, String restNSURI, String restRMPrefixResponse) {
        super();
        this.consumeSUName = consumeSUName;
        this.consumeURI = consumeURI;
        this.addrSUProvide = addrSUProvide;
        this.endpointName = endpointName;
        this.proxyName = proxyName;
        this.rest = rest;
        this.serviceName = serviceName;
        this.soap = soap;
        this.timeout = timeout;
        this.wsdl = wsdl;
        this.wsdlFile = wsdlFile;
        this.restConsumeSUName = restConsumeSUName;
        this.restConsumeURI = restConsumeURI;
        this.restNSPrefix = restNSPrefix;
        this.restNSURI = restNSURI;
        this.restRMPrefixResponse = restRMPrefixResponse;
    }

    /**
     * Allow to switch the SU in the list
     * 
     * @param sus
     * @return
     */
    private List<SuBeanForSa> switchSu(List<SuBeanForSa> sus) {
        List<SuBeanForSa> susList = new ArrayList<SuBeanForSa>();
        susList.add(sus.get(1));
        susList.add(sus.get(0));
        return susList;
    }

    /**
     * Return the new wsdlFile with the new address soap for the provide SU
     * 
     * @return
     * @throws IOException
     */
    private File modifySoapAddressWSDL() throws IOException, WSDLException, URISyntaxException {
        final String fileName = wsdlFile.getName().substring(0, wsdlFile.getName().indexOf("."));
        final File wsdlProvide = new File(repository.getCanonicalPath() + "/" + fileName
                + "Provide.wsdl");
        if (!wsdlProvide.exists()) {
            wsdlProvide.createNewFile();
        }
        FilePermissionHelper.setRights(wsdlProvide, new int[] { 7, 7, 7 }, false);

        // parse with easy wsdl
        final WSDLReader wsdlReader = WSDLFactory.newInstance().newWSDLReader();
        final Description description = wsdlReader.readWSDL(new URI(wsdlFile.toURI().toString()));
        for (final Service service : description.getServices()) {
            for (final Endpoint endpoint : service.getEndpoints()) {
                endpoint.setAddress(addrSUProvide);
            }
        }

        // writer the new wsdl file
        final WSDLWriter wsdlWriter = WSDLFactory.newInstance().newWSDLWriter();
        /*final OutputStream out = new FileOutputStream(wsdlProvide);
        out.write(wsdlWriter.writeWSDL(description).getBytes());
        out.flush();
        out.close();*/
        writeXmlFile(wsdlWriter.getDocument(description), wsdlProvide);
        
        return wsdlProvide;
        /*
         * TODO: flush document description in wsdlProvide instead
         * write with OutputStream
         */
        // Document document = wsdlWriter.getDocument(description);

    }
    
    // This method writes a DOM document to a file
    private static void writeXmlFile(Document doc, File file) {
        try {
            // Prepare the DOM document for writing
            Source source = new DOMSource(doc);
    
            // Prepare the output file
            Result result = new StreamResult(file);
    
            // Write the DOM document to the file
            Transformer xformer = TransformerFactory.newInstance().newTransformer();
            xformer.transform(source, result);
        } catch (TransformerConfigurationException e) {
        } catch (TransformerException e) {
        }
    }
    
    private static String findHostAddr(String soapAddr) throws UnknownHostException{
        String[] subString = soapAddr.split("/");

        InetAddress inetAddr = null;
        String addrport = subString[2];
        Pattern pattern = Pattern.compile(":");
        Matcher matcher = pattern.matcher(addrport);
        if(matcher.find()){
            String hostname = (String) addrport.subSequence(0, addrport.indexOf(":"));
            pattern = Pattern.compile("([0-9]|[0-9]|[0-9]).([0-9]|[0-9]|[0-9]).([0-9]|[0-9]|[0-9]).([0-9]|[0-9]|[0-9])");
            matcher = pattern.matcher(hostname);
            if(!matcher.find()){
                inetAddr = InetAddress.getByName(hostname);
            }
        }
        if(inetAddr != null){
            return inetAddr.getHostAddress();
        } else {
            return null;
        }
    }
    
    private SuBean createSOAPProvideSU(){
        
        ArrayList<XmlElement> componentElementsProvider = new ArrayList<XmlElement>();
        
        /*
         * Cut the LocalName and Namespace form the QName to avoid duplication
         * in jbi consume service-name
         */
        String LocalName = serviceName
                .substring(serviceName.indexOf("}") + 1, serviceName.length());

        /*
         * Begin Provide SOAP parameter
         */
        XmlElement element = new XmlElement();
        element.setName("soap:address");
        element.setValue(soapAddr);
        /* test de résolution d'address */
        if(soapAddr != null){
            try{
                String inetAddr = findHostAddr(soapAddr);
                if(inetAddr != null){
                    element.setValue(inetAddr);
                }
            } catch(java.net.UnknownHostException e){
                e.printStackTrace();
            }
        } else{
            element.setValue(this.endpointNameProvider);
        }
        /* fin résolution d'address*/
        componentElementsProvider.add(element);



        element = new XmlElement();
        element.setName("soap:soap-version");
        element.setValue(wsdlVersion);
        componentElementsProvider.add(element);

        element = new XmlElement();
        element.setName("soap:add-root");
        element.setValue("false");
        componentElementsProvider.add(element);

        element = new XmlElement();
        element.setName("soap:chunked-mode");
        element.setValue("false");
        componentElementsProvider.add(element);

        element = new XmlElement();
        element.setName("soap:cleanup-transport");
        element.setValue("true");
        componentElementsProvider.add(element);

        element = new XmlElement();
        element.setName("soap:mode");
        element.setValue(SOAPMODE);
        componentElementsProvider.add(element);
        /*
         * Ending Provide SOAP parameter
         */

        try {
           this.wsdlProvideFile = modifySoapAddressWSDL();
        } catch (final IOException e1) {
            System.out.println("Error occured durring the creation of the new wsdl"
                    + " for the provide: " + e1.getMessage());
        } catch (WSDLException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        } catch (URISyntaxException e) {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }

        // Create the Consumer/Provider SU
        
        SuBean suProvide = SuCreator.createSuBeanProvide(interfaceNameProvider,
                interfaceNameNsUrlProvider, LocalName, interfaceNameNsUrlProvider, addrSUProvide,
                "whatever", interfaceNameProvider, interfaceNameProvider, interfaceNameProvider,
                namespaceProvider, "operation", timeout, componentElementsProvider,this.wsdlProvideFile
                        .getName());
        return suProvide;
    }
    
    private SuBean createSOAPConsumeSU(){
        
        final ArrayList<XmlElement> componentElementsConsumer = new ArrayList<XmlElement>();
        
        final String LocalName = serviceName
        .substring(serviceName.indexOf("}") + 1, serviceName.length());
        
        XmlElement element = new XmlElement();
        element.setName("soap:address");
        element.setValue(this.consumeURI);
        componentElementsConsumer.add(element);
        
        SuBean suConsume = SuCreator.createSuBeanConsume(interfaceNameProvider,
                interfaceNameNsUrlProvider, LocalName, interfaceNameNsUrlProvider, addrSUProvide,
                "whatever", consumeURI, consumeURI, consumeURI, "ns", "operation",
                timeout, componentElementsConsumer);
        return suConsume;
    }
    
    private SuBean createRestConsumeSU(){
        
        SuBean suBean = null;
        
        final ArrayList<XmlElement> componentElementsConsumer = new ArrayList<XmlElement>();

        /*
         * Cut the LocalName and Namespace form the QName to avoid duplication
         * in jbi consume service-name
         */
   
        final String LocalName = serviceName
                .substring(serviceName.indexOf("}") + 1, serviceName.length());

        XmlElement element = new XmlElement();
        element.setName("soap:address");
        element.setValue(restConsumeURI);
        componentElementsConsumer.add(element);
        
        element = new XmlElement();
        element.setName("soap:mode");
        element.setValue(RESTMODE);
        componentElementsConsumer.add(element);
        
        if(this.restNSPrefix != null){
            element = new XmlElement();
            element.setName("soap:rest-add-namespace-prefix");
            element.setValue(this.restNSPrefix);
            componentElementsConsumer.add(element);
        }
        if(this.restNSURI != null){
            element = new XmlElement();
            element.setName("soap:rest-add-namespace-uri");
            element.setValue(this.restNSURI);
            componentElementsConsumer.add(element);
        }
        if(this.restRMPrefixResponse != null){
            element = new XmlElement();
            element.setName("soap:rest-remove-prefix-on-response");
            element.setValue(this.restRMPrefixResponse);
            componentElementsConsumer.add(element);
        }
        // Create the Consumer SU
        suBean = SuCreator.createSuBeanConsume(interfaceNameProvider,
                interfaceNameNsUrlProvider, LocalName, interfaceNameNsUrlProvider, addrSUProvide,
                "whatever", consumeURI, consumeURI, consumeURI, "ns", "operation",
                timeout, componentElementsConsumer);
        return suBean;
    }
}
