/**
 * PETALS - PETALS Services Platform. Copyright (c) 2008 EBM WebSourcing,
 * http://www.ebmwebsourcing.com/
 * 
 * This library is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Lesser General Public License as published by the Free
 * Software Foundation; either version 2.1 of the License, or (at your option)
 * any later version. This library is distributed in the hope that it will be
 * useful, but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser
 * General Public License for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this library; if not, write to the Free Software Foundation, Inc.,
 * 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
 * 
 * -------------------------------------------------------------------------
 * $Id$
 * -------------------------------------------------------------------------
 */

package org.ow2.petals.tools.webadmin.util;

import org.apache.log4j.Level;
import org.apache.log4j.Logger;

import org.w3c.dom.Document;
import org.w3c.dom.Element;
import org.w3c.dom.NamedNodeMap;
import org.w3c.dom.Node;
import org.w3c.dom.NodeList;

/**
 * @autor mbeldame - EBM Websourcing
 * 
 */

public class WebConsoleSecurityManager {
    private final Document document;
    
    private static final Logger log = Logger.getLogger(WebConsoleSecurityManager.class);
    
    private static final String securityFile = "webconsole-users.xml";

    public WebConsoleSecurityManager() {
        try
        {
            this.document = ConfigHelper.getConfig(securityFile);
        }
        catch (WebConsoleConfigException e)
        {
            log.log(Level.ERROR, "IMpossible to load the security file.");
            throw new RuntimeException(e);
        }
        log.log(Level.DEBUG,"Trying to read secutity from document " +document);

    }

    public Node getComponent(String component) throws WebConsoleConfigException {
        final NodeList list = this.document.getElementsByTagName(component);
        if (list.getLength() > 1) {
            throw new WebConsoleConfigException("Multiple " + component + " nodes found ");
        }
        if (list.getLength() < 1) {
            throw new WebConsoleConfigException("No " + component + " node found");
        }
        return list.item(0);
    }



   public Node getNodeFromProperty(String component, String nodeName, String property,
            String propertyValue) throws WebConsoleConfigException {
        final Node result = null;
        final Node racine = this.getComponent(component);
        final NodeList nodeList = racine.getChildNodes();
        for (int i = 0; i < nodeList.getLength(); i++) {
            final Node n = nodeList.item(i);
            if (n instanceof Element) {
                if (n.getNodeName().equalsIgnoreCase(nodeName)) {
                    final NamedNodeMap list = n.getAttributes();
                    for (int a = 0; a < list.getLength(); a++) {
                        if ((property.equalsIgnoreCase(list.item(a).getNodeName()))
                                && (propertyValue.equalsIgnoreCase(list.item(a).getNodeValue()))) {
                            return n;
                        }
                    }
                }
            }
        }
        return result;
    }

}
